"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_1 = require("./generator");
const schema_1 = require("./schema");
const utils_1 = require("../utils");
class ApiGenerator extends generator_1.FileGenerator {
    constructor(schemaData, file, type, apiTypesFilename) {
        super(file);
        this.schemaData = schemaData;
        this.type = type;
        this.apiTypesFilename = apiTypesFilename;
    }
    getContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const methods = this.type === "mutation"
                ? this.generateMutationMethods()
                : this.generateQueryMethods();
            const api = this.generateApi(this.type);
            return [api.data, methods.data].join("\n\n");
        });
    }
    generateMutationMethods() {
        const schema = this.schemaData.schema;
        let type;
        if (schema.data.__schema["mutationType"]) {
            const typeName = schema.data.__schema["mutationType"].name;
            type = this.schemaData.getTypeByName(typeName);
        }
        return generateMethods("mutation", type);
    }
    generateQueryMethods() {
        const schema = this.schemaData.schema;
        const typeName = schema.data.__schema["queryType"].name;
        const type = this.schemaData.getTypeByName(typeName);
        return generateMethods("query", type);
    }
    generateApi(action) {
        return generateApi(action, this.schemaData, this.apiTypesFilename);
    }
}
exports.ApiGenerator = ApiGenerator;
function generateApi(action, schemaData, apiTypesFilename) {
    const schema = schemaData.schema;
    const typeExists = !!schema.data.__schema[action + "Type"];
    const type = typeExists
        ? schemaData.getTypeByName(schema.data.__schema[action + "Type"].name)
        : { fields: [] };
    const upperAction = utils_1.startWithUpperCase(action);
    const importedTypes = [];
    let hasDataField = false;
    const methods = type.fields.map((field) => {
        const methodName = field.name.includes("_")
            ? field.name.split("_")[0] + utils_1.startWithUpperCase(field.name.split("_")[1])
            : field.name;
        const methodArgsData = [{ name: "key", type: "keyof T" }];
        const resultIsObject = schemaData.typeIsObject(field.type) ||
            (schema_1.typeIsList(field.type) &&
                schemaData.typeIsObject(field.type.ofType.ofType));
        if (resultIsObject) {
            methodArgsData.push({ name: "data", type: "GraphQlQueryItemInput" });
            hasDataField = true;
        }
        const argData = {
            name: "args",
            type: schema_1.generateTypeScriptType(field.args.map((arg) => {
                const typeName = schema_1.getJsTypeName(arg.type);
                if (schemaData.typeIsObject(arg.type)) {
                    importedTypes.push(schema_1.getTypeName(arg.type));
                }
                return {
                    name: arg.name,
                    type: typeName,
                    required: schema_1.typeIsRequired(arg.type)
                };
            }))
        };
        if (field.args && field.args.length) {
            methodArgsData.push(argData);
        }
        const resultJsType = schema_1.getJsTypeName(field.type);
        if (schemaData.typeIsObject(field.type)) {
            importedTypes.push(schema_1.getTypeName(field.type));
        }
        methodArgsData.push({
            name: "mapper?",
            type: `IDataMapper<MR, ${resultJsType}>`
        });
        const methodArgs = methodArgsData.map(item => `${item.name}:${item.type}`);
        const variables = field.args.map((arg) => {
            let typeName = schema_1.getTypeName(arg.type);
            if (schema_1.typeIsList(arg.type)) {
                if (arg.type.ofType &&
                    arg.type.ofType.ofType &&
                    schema_1.typeIsRequired(arg.type.ofType.ofType)) {
                    typeName += "!";
                }
                typeName = `[${typeName}]`;
            }
            if (schema_1.typeIsRequired(arg.type)) {
                typeName += "!";
            }
            return `{ name: '${arg.name}', value: args.${arg.name}, type: '${typeName}' }`;
        });
        const methodBody = `return this._client.queryAddItem(key,
            {
                ${resultIsObject ? "fields: data.fields," : ""}
                name: ${upperAction}Methods.${field.name},
                mapper: mapper,
                variables: [
                    ${variables.join(",\n")}
                ]
            })`;
        return `${methodName}<MR>(${methodArgs.join(",\n")}) {
        
        ${methodBody}
    }`;
    });
    const data = `
import { ${utils_1.uniq(importedTypes).join(", ")} } from './${apiTypesFilename}';
import { GraphQlRequestResult, GraphQlQuery, IGraphQlQueryExecutor, ${hasDataField ? "GraphQlQueryItemInput," : ""} IDataMapper } from 'graphql-client-ts';

export class ${upperAction}Api<T> {
    protected _client: GraphQlQuery<T, ${upperAction}Methods>;
    constructor(executor: IGraphQlQueryExecutor) {
        this._client = new GraphQlQuery<T, ${upperAction}Methods>(executor, '${action}');
    }
    queryHasItems() {
        return this._client.queryHasItems();
    }
    async queryExecute(): Promise<GraphQlRequestResult<T>> {
        return this._client.queryExecute();
    }
    ${methods.join("\n\n")}
}
    `;
    return {
        name: `${upperAction}Api`,
        data
    };
}
function generateMethods(action, type) {
    const names = type
        ? type.fields.map(field => `    ${field.name} = "${field.name}"`)
        : [];
    const name = `${utils_1.startWithUpperCase(action)}Methods`;
    const data = `export enum ${name} {
${names.join(",\n")}
}`;
    return { name, data };
}
