"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const got = require("got");
const url_1 = require("url");
class SchemaData {
    constructor(schema) {
        this.schema = schema;
    }
    getTypeByName(name) {
        return this.schema.data.__schema.types.find((type) => type.name === name);
    }
    typeIsObject(type) {
        return this.isOfType(type, ["OBJECT", "INPUT_OBJECT"]);
    }
    typeIsObjectOrEnum(type) {
        return this.isOfType(type, ["OBJECT", "INPUT_OBJECT", "ENUM"]);
    }
    typeIsEnum(type) {
        return this.isOfType(type, ["ENUM"]);
    }
    isOfType(type, types) {
        const schema = this.schema;
        const invalidNames = [schema.data.__schema["queryType"].name];
        if (schema.data.__schema["mutationType"]) {
            invalidNames.push(schema.data.__schema["mutationType"].name);
        }
        const name = getTypeName(type);
        const kind = getTypeKind(type);
        return (name.indexOf("__") !== 0 &&
            invalidNames.indexOf(name) < 0 &&
            types.indexOf(kind) > -1);
    }
    static create(url, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!url) {
                throw new Error(`url is required!`);
            }
            headers = headers || {};
            headers["content-type"] = headers["content-type"] || "application/json";
            const parsedUrl = url_1.parse(url);
            headers["origin"] =
                headers["origin"] || `${parsedUrl.protocol}//${parsedUrl.hostname}`;
            const response = yield got(url, {
                headers,
                body: { query: schemaQuery, variables: null },
                method: "POST",
                json: true
            });
            if (response.statusCode >= 400) {
                const data = yield response.body;
                throw new Error(`Bad response(${response.statusCode}) from server: ${data}`);
            }
            const schema = yield response.body;
            return new SchemaData(schema);
        });
    }
}
exports.SchemaData = SchemaData;
const schemaQuery = "\n    query IntrospectionQuery {\n      __schema {\n        queryType { name }\n        mutationType { name }\n        subscriptionType { name }\n        types {\n          ...FullType\n        }\n        directives {\n          name\n          description\n          locations\n          args {\n            ...InputValue\n          }\n        }\n      }\n    }\n\n    fragment FullType on __Type {\n      kind\n      name\n      description\n      fields(includeDeprecated: true) {\n        name\n        description\n        args {\n          ...InputValue\n        }\n        type {\n          ...TypeRef\n        }\n        isDeprecated\n        deprecationReason\n      }\n      inputFields {\n        ...InputValue\n      }\n      interfaces {\n        ...TypeRef\n      }\n      enumValues(includeDeprecated: true) {\n        name\n        description\n        isDeprecated\n        deprecationReason\n      }\n      possibleTypes {\n        ...TypeRef\n      }\n    }\n\n    fragment InputValue on __InputValue {\n      name\n      description\n      type { ...TypeRef }\n      defaultValue\n    }\n\n    fragment TypeRef on __Type {\n      kind\n      name\n      ofType {\n        kind\n        name\n        ofType {\n          kind\n          name\n          ofType {\n            kind\n            name\n            ofType {\n              kind\n              name\n              ofType {\n                kind\n                name\n                ofType {\n                  kind\n                  name\n                  ofType {\n                    kind\n                    name\n                  }\n                }\n              }\n            }\n          }\n        }\n      }\n    }\n  ";
function saveCodeFile(data, fileName) {
    const content = data.map(item => item.data).join("\n\n");
    fs_1.writeFileSync(path_1.join(__dirname, "..", "..", "src", fileName + ".ts"), content, "utf8");
}
exports.saveCodeFile = saveCodeFile;
function generateTypeScriptType(items) {
    return `{ ${items
        .map(item => `${item.name}${item.required ? "" : "?"}: ${item.type}`)
        .join(", ")} } `;
}
exports.generateTypeScriptType = generateTypeScriptType;
function generateEnumScript(items) {
    return `{ ${items.map(item => `${item.name} = "${item.name}"`).join(", ")} } `;
}
exports.generateEnumScript = generateEnumScript;
function getJsTypeName(type) {
    const name = getTypeName(type);
    let jsName = name;
    switch (name) {
        case "ID":
        case "String":
            jsName = "string";
            break;
        case "Int":
        case "Float":
            jsName = "number";
            break;
        case "JSON":
            jsName = "any";
            break;
        case "Boolean":
            jsName = "boolean";
            break;
    }
    if (typeIsList(type)) {
        jsName += "[]";
    }
    return jsName;
}
exports.getJsTypeName = getJsTypeName;
function typeIsList(type) {
    return ((type.kind === "NON_NULL" && type.ofType.kind === "LIST") ||
        type.kind === "LIST");
}
exports.typeIsList = typeIsList;
function typeIsRequired(type) {
    return type.kind === "NON_NULL";
}
exports.typeIsRequired = typeIsRequired;
function getTypeName(type) {
    if (~["NON_NULL", "LIST"].indexOf(type.kind)) {
        return getTypeName(type.ofType);
    }
    return type.name;
}
exports.getTypeName = getTypeName;
function getTypeKind(type) {
    if (~["NON_NULL", "LIST"].indexOf(type.kind)) {
        return getTypeKind(type.ofType);
    }
    return type.kind;
}
exports.getTypeKind = getTypeKind;
